@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: approve.c,v 1.104 1998/05/19 18:38:45 will Exp $";
+static char rcsid[] = "$Id: approve.c,v 1.105 1999/02/18 20:31:01 will Exp $";
 
 /* The code should compile with either ANSI C or K&R compilers. */
 
@@ -744,12 +744,12 @@ int verbosity;
 	    }
 	    if (havewildcard) {
 		for (i=0; i<localinfo.progs.n-1; i++) {
-		    printf("%s -> %s;\n", prog,
+		    printf("%s: %s -> %s;\n", prog,
 			localinfo.progs.cmd_file[i].Cmd,
 			localinfo.progs.cmd_file[i].File);
 		}
 		i = localinfo.progs.n - 1;
-		printf("%s -> %s:\n", prog,
+		printf("%s: %s -> %s:\n", prog,
 		    localinfo.progs.cmd_file[i].Cmd,
 		    localinfo.progs.cmd_file[i].File);
 		if (localinfo.info && *localinfo.info)
@@ -857,12 +857,12 @@ int verbosity;
     if (localinfo.maxlen1arg < 0)
 	(void) printf("\tNo per-arg length limit; ");
     else
-	(void) printf("\tMax per-arg length: %d chars; ", localinfo.maxlen1arg);
+	(void) printf("\tMax per-arg length: %ld chars; ",localinfo.maxlen1arg);
 
     if (localinfo.maxlenargs < 0)
 	(void) printf("no limit on sum of all args.\n");
     else
-	(void) printf("max over all args: %d chars.\n", localinfo.maxlenargs);
+	(void) printf("max over all args: %ld chars.\n", localinfo.maxlenargs);
 
     n = StrLastInUse(&localinfo.argpats);
     if (n >= 0) {
