@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: checks.c,v 1.53 1998/04/14 17:06:38 will Exp $";
+static char rcsid[] = "$Id: checks.c,v 1.60 1999/02/26 08:36:44 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -53,6 +53,66 @@ char *pat;	/* pattern that value must match */
     return 0;
 }
 
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Compare a value to a pattern */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Returns 0 if pat matched; -1 otherwise.  */
+int
+check_value(value, pat)
+char *value;	/* value to check */
+char *pat;	/* pattern that value must match */
+{
+
+    if (!value)
+	return -1;
+
+    if (debug)
+	(void) fprintf(stderr,
+	    "\tcheck_value args: value=\"%s\"; pat=\"%s\"\n",
+	    value, pat);
+    
+    if (s_re_comp(pat))
+	return Error(0, 0,
+	    "%t\n\tcheck_value(): couldn't compile pattern `%-.500s'.\n", pat);
+
+    if (s_re_exec(value) != 1)
+	return Error(0, 0,
+	    "check_value(): value doesn't match pattern %-.500s.\n", pat);
+    return 0;
+}
+
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Option checking -- ensure that options to super are reasonable strings. */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* Returns 0 if reasonable; -1 otherwise.  */
+int
+checkarg(str)
+char *str;
+{
+    int len;
+    static int optlen_tot = 0;	/* running length of all options to super */
+
+    /* Limit each option to a reasonable length (MAXOPTLEN chars) */
+    len = strlen(str);
+    if (len > MAXOPTLEN) {
+	return Error(0, 0, "Command-line options may not \
+exceed %d chars; you passed a string of length %d!\n", MAXOPTLEN, len);
+    }
+
+    /* Limit the combined option to length MAXOPTLEN_TOT chars */
+    optlen_tot += len;
+    if (optlen_tot > MAXOPTLEN_TOT) {
+	return Error(0, 0, "The total length of command-line options \
+may not exceed %d chars!\n", MAXOPTLEN_TOT);
+    }
+
+    /* Ensure the option pattern matches OPT_PATTERN */
+    if (check_value(str, OPT_PATTERN) != 0)
+	return -1;
+    
+    return 0;
+}
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Look up owner of a file, return uid and gid of owner */
 /* Return 0 on success, -1 & print message on failure */
@@ -178,6 +238,10 @@ set_suppl_groups()
     int i;
 
     /* Set the supplementary groups */
+    if (geteuid() != 0) {
+	/* Can't set supplementary groups if we've changed to some other uid */
+	return 0;
+    }
     if (*localinfo.u_g) {
 	/* Default is to take from that user.  (Don't worry if there is no
 	 * such user; it'll get noticed later.)  Conflicts with local
@@ -417,7 +481,6 @@ get_encrypted_pw()
 int
 get_encrypted_pw()
 {
-fprintf(stderr, "here2\n");
     /* Vanilla password file lookup */
 
     strcpy(userinfo.encr, userinfo.caller.pw_passwd);
