@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: options.c,v 1.72 1999/02/15 10:18:19 will Exp $";
+static char rcsid[] = "$Id: options.c,v 1.80 1999/02/24 06:11:16 will Exp $";
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Process a global xxx=yyy option */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -42,6 +42,8 @@ Option opts[] = {
 	{ "rlog_host",		GLOBAL,		option_rlog_host },
 	{ "setenv",		GLOBAL|LOCAL,	option_setenv },
 	{ "syslog",		GLOBAL,		option_syslog },
+	{ "syslog_success",	GLOBAL,		option_syslog_success },
+	{ "syslog_error",	GLOBAL,		option_syslog_error },
 	{ "timeout",		GLOBAL|LOCAL,	option_timeout },
 	{ "timestampbyhost",	GLOBAL,		option_timestampbyhost },
 	{ "timestampuid",	GLOBAL,		option_timestampuid },
@@ -653,9 +655,11 @@ int isglobal;
 	fprintf(stderr, "\toption:logfile=%s (%s)\n",
 				s, isglobal ? "global" : "local");
 
-    /* Don't do any logging if just checking syntax! */
-    if (check_syntax)
+    /* Don't do any logging if debugging or just checking syntax! */
+    if (check_syntax || it_came_from_cmdline) {
+	Error(0, 0, "%t\n\tIgnoring logfile option, due to debug mode\n");
 	return 0;
+    }
 
     strcpy(globalinfo.log.filename, s);
     globalinfo.log.newfile = 1;
@@ -792,7 +796,7 @@ int isglobal;
 	fprintf(stderr, "\toption:maxlen=%s (%s)\n",
 				s, isglobal ? "global" : "local");
 
-    i = sscanf(s, "%ld%,%ld", &m, &n);
+    i = sscanf(s, "%ld,%ld", &m, &n);
     switch(i) {
     case 1:
 	n = m;
@@ -871,6 +875,11 @@ int isglobal;
 in `%s' -- value must be `y' or `n'\n", word);
     }
 
+    if (required && using_user_supertab) {
+	return Error(0, 0, "%t\n\tUser-supplied .supertab files may not do \
+password checking -- sorry.\n");
+    }
+
     if (isglobal)
 	globalinfo.passinfo.required = required;
     else
@@ -972,6 +981,11 @@ int isglobal;
 	fprintf(stderr, "\toption:rlog_host=%s (%s)\n",
 				s, isglobal ? "global" : "local");
 
+#ifndef USE_RSYSLOG
+    return Error(0, 0, "%t\n\tOption rlog_host is not allowed, because \
+this copy of super was compiled with rsyslog disabled.\n");
+#endif
+
     if (*s == '\0')
 	return Error(0, 0, "%t\n\tInvalid option value in `%s' \
 -- empty value not allowed.\n", word);
@@ -1059,13 +1073,59 @@ int isglobal;
 	    return Error(0, 0, "%t\n\tInvalid option value in `%s' \
 -- value must be `y' or `n'.\n", word);
 #ifndef HAVE_SYSLOG
-	return Error(0, 0, "%t\n\tCan't enable syslog() calls -- \
+	return Error(0, 0, "%t\n\tCan't use option syslog -- \
 not compiled with HAVE_SYSLOG defined.\n");
 #endif
     }
     return 0;
 }
 
+static int
+option_syslog_error(word, s, isglobal)
+char *word;
+char *s;
+int isglobal;
+{
+    int syslogcode;
+
+    if (debug)
+	fprintf(stderr, "\toption:syslog_error=%s (%s)\n",
+				s, isglobal ? "global" : "local");
+
+#ifndef HAVE_SYSLOG
+	return Error(0, 0, "%t\n\tCan't use option syslog_error -- \
+not compiled with HAVE_SYSLOG defined.\n");
+#endif
+
+    if (read_syslogcode(s, &syslogcode) == -1)
+	return -1;
+    error_priority = syslogcode;
+    return 0;
+}
+
+static int
+option_syslog_success(word, s, isglobal)
+char *word;
+char *s;
+int isglobal;
+{
+    int syslogcode;
+
+    if (debug)
+	fprintf(stderr, "\toption:syslog_success=%s (%s)\n",
+				s, isglobal ? "global" : "local");
+
+#ifndef HAVE_SYSLOG
+	return Error(0, 0, "%t\n\tCan't use option syslog_success -- \
+not compiled with HAVE_SYSLOG defined.\n");
+#endif
+
+    if (read_syslogcode(s, &syslogcode) == -1)
+	return -1;
+    globalinfo.log.syslog_success = syslogcode;
+    return 0;
+}
+
 static int
 option_timeout(word, s, isglobal)
 char *word;
