@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: super.c,v 1.462 1999/02/15 10:18:37 will Exp $";
+static char rcsid[] = "$Id: super.c,v 1.485 1999/02/26 08:44:33 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -117,7 +117,8 @@ GlobalInfo globalinfo = {
 	    0,		/* uid: UID under which we open logfile */
 	    -1,		/* pid: PID of the logger process */
 	    0,		/* newfile: !0 if logfile given but not yet used */
-	    0 },	/* newuid: !0 if loguid given but not yet used */
+	    0,		/* newuid: !0 if loguid given but not yet used */
+	    LOG_INFO },	/* syslog() priority for success message */
 
 	"",		/* mailcmd */
 	0,		/* mail_success (bool) */
@@ -163,6 +164,7 @@ char *prog;				/* this program */
 int debug=0;				/* Set by the debug options flag */
 int it_came_from_cmdline=0;		/* Set by -F/-T/-U/-G/-M flags */
 int check_syntax=0;			/* Set by the -c options flag */
+int using_user_supertab = 0;		/* !0 means using a user's .supertab */
 
 /* Routines used to compile/match user/group/host patterns */
 char *(*pat_compile) P__((char *)) = s_re_comp;
@@ -265,11 +267,13 @@ char **argv;
 	Error(0,0, "Checking syntax of superfile `%s'\n", superfile);
 
     /* Check if we need to switch to processing a user's super file */
+    using_user_supertab = 0;
     if (cmd && strchr(cmd, ':')) {
 	char *tcmd = strchr(cmd, ':') + 1;
 	if (user_supertab(cmd) == -1)
 	    return 1;
 	cmd = tcmd;
+	using_user_supertab = 1;
     }
 
     /* Print version if explicitly requested or if giving help */
@@ -328,7 +332,12 @@ char **argv;
     }
 
     /* Set uid/gid if necessary */
-    status = set_u_g();
+    if (using_user_supertab) {
+	/* Already changed to user's uid/gid earlier */
+	status = 0;
+    } else {
+	status = set_u_g();
+    }
 
     /* Button up for security, and get a modified environment */
     envp = buttonup(cmd);
@@ -374,7 +383,7 @@ due to previously-reported problem)\n");
 #ifdef HAVE_SYSLOG
 	extern int error_priority;
 	int old_pri = error_priority;
-	error_priority = LOG_INFO;
+	error_priority = globalinfo.log.syslog_success;
 #endif
 	logmsg(cmd, arglist);
 #ifdef HAVE_SYSLOG
@@ -455,9 +464,11 @@ char **argv;
 int *givehelp, *giveversion, *verbosity, *printvars;
 {
     int iarg;
-    char *p, *s, *t;
+    char *s, *t;
     short minutes;
+#ifndef HAVE_LOCALTIME
     char daynum;
+#endif
     int takes_optional_superfile;
     long l;
     struct passwd *pw;
@@ -472,6 +483,10 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 	    iarg++;
 	    break;
 	}
+
+	if (checkarg(argv[iarg]) != 0)
+	    return -1;
+
 	for (s = &argv[iarg][1]; *s; s++) {
 	    takes_optional_superfile = 0;
 	    switch (*s) {
@@ -492,6 +507,8 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
 		    /* User gave -G gid */
 		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
 		    t = argv[iarg];
 		} else {
 		    return Error(0, 0, "No gid specified after option -G\n");
@@ -501,6 +518,7 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 		    return Error(0,0,"Invalid gid specified after option -G\n");
 		else
 		    userinfo.orig_gid = l;
+		    userinfo.caller.pw_gid = l;
 		break;
 	    case 'M':
 		it_came_from_cmdline = 1;
@@ -511,6 +529,8 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
 		    /* User gave -M mach */
 		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
 		    t = argv[iarg];
 		} else {
 		    return Error(0, 0, "No machine specified after option -M\n");
@@ -534,6 +554,8 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
 		    /* User gave -T time */
 		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
 		    t = argv[iarg];
 		} else {
 		    return Error(0, 0, "No time specified after option -T\n");
@@ -551,15 +573,34 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
 		    /* User gave -U uid */
 		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
 		    t = argv[iarg];
 		} else {
 		    return Error(0, 0, "No uid specified after option -U\n");
 		}
 		pw = getpwentry(1, t);
-		if (!p)
+		if (!pw) {
 		    return -1;
-		else
+		} else {
+		    /* Act as if we really were the named user */
+		    userinfo.orig_uid = pw->pw_uid;
 		    userinfo.orig_gid = pw->pw_gid;
+		    userinfo.caller.pw_uid = pw->pw_uid;
+		    userinfo.caller.pw_gid = pw->pw_gid;
+		    if (!(userinfo.caller.pw_name = strdup(pw->pw_name)))
+			(void) Error(0, 2,
+			  "failed to malloc space for passwd struct field.\n");
+		    if (!(userinfo.caller.pw_passwd = strdup(pw->pw_passwd)))
+			(void) Error(0, 2,
+			  "failed to malloc space for passwd struct field.\n");
+		    if (!(userinfo.caller.pw_dir = strdup(pw->pw_dir)))
+			(void) Error(0, 2,
+			  "failed to malloc space passwd struct field.\n");
+
+		    fprintf(stderr, "\n\tActing as if uid=%d (%s), gid=%d\n\n",
+		    			pw->pw_uid, pw->pw_name, pw->pw_gid);
+		}
 		break;
 	    case 'b': *printvars = 1; break;
 	    case 'd':
@@ -579,19 +620,45 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 	    /* Check for optional superfile */
 	    if (takes_optional_superfile) {
 		if (*(s+1)) {
-		    /* User gave -Xfile (where X is -F/-T/-U/-G/-M) */
+		    /* User gave -Xfile (where X is -c/-F) */
 		    superfile = s+1;
 		    s += strlen(s) - 1;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
 		    /* User gave -X file */
 		    iarg++;
+		    if (checkarg(argv[iarg]) != 0)
+			return -1;
 		    superfile = argv[iarg];
+		} else {
+		    /* No file was given.  Use the default super file */
 		}
 		/* Verify that this user has permission to access the file */
 		if (access(superfile, R_OK) == -1)
 		    Error(1, 1, "You can't read `%s': ", superfile);
-		else
 		error_srcfile = superfile;
+
+		if (strcmp(superfile, SUPERFILE) != 0) {
+		    /* User has selected a different file.  Don't allow
+		     * data-driven attacks via a user-supplied superfile:
+		     * if our real uid isn't root, reset our effective uid
+		     * to the real uid.
+		     */
+		    if (getuid() != 0) {
+			setuid(getuid());
+			fprintf(stderr,
+    "\t** Since you have supplied a super.tab file that isn't the default,\n");
+			fprintf(stderr,
+    "\t** and your real uid isn't root, we're going to change back to your\n");
+			fprintf(stderr,
+    "\t** real uid for this test.  That protects us against attacks via\n");
+			fprintf(stderr,
+    "\t** nasty constructions inside user-supplied super.tab files.\n");
+			fprintf(stderr,
+    "\t** Not that we don't trust you...\n\n");
+			fprintf(stderr,
+    "\t** Now using: ruid=%d  euid=%d **\n\n", getuid(), geteuid());
+		    }
+		}
 	    }
 	}
     }
@@ -816,17 +883,17 @@ char *command;		/* user:cmd */
 
     if (initgroups(user, pass->pw_gid) == -1)
 	return Error(0, 0,
-	    "Couldn't set groups to those of user %s (from command <%s:%s>).\n",
+	"Can't set groups to those of user %s (from command <%s:%-.500s>).\n",
 	    user, user, cmd);
 
     if (setgid(pass->pw_gid) == -1)
 	return Error(1, 0,
-	    "Couldn't set gid to that of user %s (from command <%s:%s>): ",
+	    "Can't set gid to that of user %s (from command <%s:%-.500s>): ",
 	    user, user, cmd);
 
     if (setuid(pass->pw_uid) == -1)
 	return Error(1, 0,
-	    "Couldn't set uid to user %s (from command <%s:%s>): ",
+	    "Can't set uid to user %s (from command <%s:%-.500s>): ",
 	    user, user, cmd);
     return 0;
 }
@@ -1124,7 +1191,7 @@ char *cmd;		/* name of command being started */
 	s = Getenv(*p);
 	if (s) {
 	    env[j] = s;
-	    if (strlen(env[i])+1 > localinfo.maxenvlen) {
+	    if (strlen(env[j])+1 > localinfo.maxenvlen) {
 		Error(0, 0, "%t\n\tDefinition for envvar %s exceeds \
 maxenvlen=%d.\n", *p, localinfo.maxenvlen);
 		return NULL;
@@ -1245,6 +1312,9 @@ init_globalinfo()
 
     for (i=0; i<=MAXSETENV; i++)
 	globalinfo.setenv[i]=NULL;
+
+    error_priority = LOG_ERR;
+    error_facility = LOG_USER;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -1270,6 +1340,7 @@ init_localinfo()
     localinfo.file_uid = UID_NOTSET;
     localinfo.file_gid = GID_NOTSET;
     localinfo.nice_incr = globalinfo.nice_incr;
+	       
     /* Don't init to global ngroups, because we need to be able to tell
      * later on whether local groups=xxx or global groups=xxx was used.
      */
@@ -1320,18 +1391,12 @@ init_userinfo()
     /* Since the string fields that we need are overwritten by later
      * calls to getpwxxx(), make private copies:
      */
-    if (!(s = malloc(strlen(userinfo.caller.pw_name)+1)))
+    if (!(userinfo.caller.pw_name = strdup(usrpw->pw_name)))
 	(void) Error(0, 2, "failed to malloc space for passwd struct field.\n");
-    strcpy(s, userinfo.caller.pw_name);
-    userinfo.caller.pw_name = s;
-    if (!(s = malloc(strlen(userinfo.caller.pw_passwd)+1)))
+    if (!(userinfo.caller.pw_passwd = strdup(usrpw->pw_passwd)))
 	(void) Error(0, 2, "failed to malloc space for passwd struct field.\n");
-    strcpy(s, userinfo.caller.pw_passwd);
-    userinfo.caller.pw_passwd = s;
-    if (!(s = malloc(strlen(userinfo.caller.pw_dir)+1)))
+    if (!(userinfo.caller.pw_dir = strdup(usrpw->pw_dir)))
 	(void) Error(0, 2, "failed to malloc space passwd struct field.\n");
-    strcpy(s, userinfo.caller.pw_dir);
-    userinfo.caller.pw_dir = s;
     
     error_user = userinfo.caller.pw_name;
     userinfo.orig_mask = umask(022); /* Get orig umask, and set to 022 */
